import { loggerContext } from "./platform/logger-context";
import { errorToException } from "./util/error-util";
import { APIException, JavaScriptException, NetworkConnectionException } from "./Exception";
import { app } from "./app";
export class LoggerImpl {
    constructor() {
        this.environmentContext = {};
        this.logQueue = [];
        this.environmentContext = loggerContext;
    }
    addContext(context) {
        this.environmentContext = Object.assign(Object.assign({}, this.environmentContext), context);
    }
    info(action, info, elapsedTime) {
        this.appendLog("OK", { action, info, elapsedTime: elapsedTime || 0 });
    }
    warn(data) {
        this.appendLog("WARN", data);
    }
    error(data) {
        this.appendLog("ERROR", data);
    }
    exception(exception, extra, action) {
        let isWarning;
        let errorCode;
        const info = Object.assign({}, extra);
        if (exception instanceof NetworkConnectionException) {
            isWarning = true;
            errorCode = "NETWORK_FAILURE";
            info["requestURL"] = exception.requestURL;
            info["originalErrorMessage"] = exception.originalErrorMessage;
        }
        else if (exception instanceof APIException) {
            if (exception.statusCode === 400 && exception.errorCode === "VALIDATION_ERROR") {
                isWarning = false;
                errorCode = "API_VALIDATION_ERROR";
            }
            else {
                isWarning = true;
                errorCode = `API_ERROR_${exception.statusCode}`;
            }
            info["requestURL"] = exception.requestURL;
            info["responseData"] = JSON.stringify(exception.responseData);
            if (exception.errorId) {
                info["apiErrorId"] = exception.errorId;
            }
            if (exception.errorCode) {
                info["apiErrorCode"] = exception.errorCode;
            }
        }
        else if (exception instanceof JavaScriptException) {
            isWarning = false;
            errorCode = "JAVASCRIPT_ERROR";
            info["appState"] = JSON.stringify(app.store.getState().app);
        }
        else {
            console.warn("[framework] Exception class should not be extended, throw Error instead");
            isWarning = false;
            errorCode = "JAVASCRIPT_ERROR";
        }
        // isWarning = isWarning || !shouldAlertToUser(exception.message);
        this.appendLog(isWarning ? "WARN" : "ERROR", { action, errorCode, errorMessage: exception.message, info, elapsedTime: 0 });
    }
    collect() {
        return this.logQueue;
    }
    empty() {
        this.logQueue = [];
    }
    appendLog(result, data) {
        const completeContext = {};
        Object.entries(this.environmentContext).map(([key, value]) => {
            if (typeof value === "string") {
                completeContext[key] = value.substr(0, 1000);
            }
            else {
                try {
                    completeContext[key] = value();
                }
                catch (e) {
                    const message = errorToException(e).message;
                    completeContext[key] = "[error] " + message;
                    console.warn("[framework] Fail to execute logger context: " + message);
                }
            }
        });
        const event = Object.assign({ date: new Date(), result, context: completeContext }, data);
        this.logQueue.push(event);
    }
}
//# sourceMappingURL=Logger.js.map