import { __awaiter } from "tslib";
import React from "react";
import { AppRegistry, AppState } from "react-native";
import { Provider } from "react-redux";
import { app } from "../app";
import { call, delay } from "../typed-saga";
import { ErrorBoundary } from "../util/ErrorBoundary";
import { ajax } from "../util/network";
import { APIException } from "../Exception";
import { captureError } from "../util/error-util";
const LOGGER_ACTION = "@@framework/logger";
export function startApp(config) {
    setupGlobalErrorHandler(config.errorListener);
    runBackgroundLoop(config.logger);
    renderRoot(config.registeredAppName, config.componentType, config.beforeRendering);
}
function setupGlobalErrorHandler(errorListener) {
    app.errorHandler = errorListener.onError.bind(errorListener);
    ErrorUtils.setGlobalHandler((error, isFatal) => captureError(error, "@@framework/global", { severity: isFatal ? "fatal" : undefined }));
}
function renderRoot(registeredAppName, EntryComponent, beforeRendering) {
    class WrappedAppComponent extends React.PureComponent {
        constructor(props) {
            super(props);
            this.onAppStateChange = (nextAppState) => {
                const { appState } = this.state;
                if (["inactive", "background"].includes(appState) && nextAppState === "active") {
                    app.logger.info("@@ACTIVE", { prevState: appState });
                }
                else if (appState === "active" && ["inactive", "background"].includes(nextAppState)) {
                    app.logger.info("@@INACTIVE", { nextState: nextAppState });
                }
                this.setState({ appState: nextAppState });
            };
            this.state = { initialized: false, appState: AppState.currentState };
        }
        componentDidMount() {
            return __awaiter(this, void 0, void 0, function* () {
                if (beforeRendering) {
                    yield beforeRendering();
                }
                this.setState({ initialized: true });
                AppState.addEventListener("change", this.onAppStateChange);
            });
        }
        componentWillUnmount() {
            AppState.removeEventListener("change", this.onAppStateChange);
        }
        render() {
            return (this.state.initialized && (<Provider store={app.store}>
                        <ErrorBoundary>
                            <EntryComponent />
                        </ErrorBoundary>
                    </Provider>));
        }
    }
    AppRegistry.registerComponent(registeredAppName, () => WrappedAppComponent);
}
function runBackgroundLoop(loggerConfig) {
    app.logger.info("@@ENTER", {});
    app.loggerConfig = loggerConfig || null;
    app.sagaMiddleware.run(function* () {
        while (true) {
            // Loop on every 30 second
            yield delay(30000);
            // Send collected log to event server
            yield* call(sendEventLogs);
        }
    });
}
export function sendEventLogs() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const loggerConfig = app.loggerConfig;
            if (loggerConfig) {
                const logs = app.logger.collect();
                if (logs.length > 0) {
                    yield ajax("POST", loggerConfig.serverURL, {}, { events: logs }, true);
                    app.logger.empty();
                }
            }
        }
        catch (e) {
            if (e instanceof APIException) {
                // For APIException, retry always leads to same error, so have to give up
                const length = app.logger.collect().length;
                app.logger.empty();
                app.logger.exception(e, { droppedLogs: length.toString() }, LOGGER_ACTION);
            }
        }
    });
}
//# sourceMappingURL=bootstrap.js.map